@echo off
@REM
@REM For the build environment, certain ENV vars are expected to be 
@REM setup for tool/config usage.
@REM    The ENV vars are:  TOOL_DOXYGEN, TOOL_GRAPHVIZ_DIR, CVC_BUILD_VER
@REM    The default ENV settings may be modified in this script.
@REM    The CVC_BUILD_VER can be set by passing the version as an arg.
@REM

setlocal

set VERSION=%1

@REM Get the name of the batch file and strip the full path.
for %%I in (%0) do set PROG=%%~nI%%~xI

@REM
@REM Check ENV vars and set defaults.
@REM
if "%TOOL_DOXYGEN%"=="" (
  set TOOL_DOXYGEN=%ProgramFiles%\doxygen\bin\doxygen.exe
)
if not exist "%TOOL_DOXYGEN%" (
  set "BATCH_ERR=Could not find %TOOL_DOXYGEN%"
  goto END
)

if "%TOOL_GRAPHVIZ_DIR%"=="" (
  set TOOL_GRAPHVIZ_DIR=%ProgramFiles%\Graphviz2.26.3\bin
)
if not exist "%TOOL_GRAPHVIZ_DIR%" (
  set "BATCH_ERR=Could not find %TOOL_GRAPHVIZ_DIR%"
  goto END
)

if not "%VERSION%"==""  set CVC_BUILD_VER=%VERSION%
if "%CVC_BUILD_VER%"=="" (
  set "BATCH_ERR=CVC_BUILD_VER is not set"
  goto END
)


echo Generating from 'ApiDoxyConfig'...
"%TOOL_DOXYGEN%" ApiDoxyConfig
if errorlevel 1 (
  set "BATCH_ERR=Doxygen tool failed - ApiDoxyConfig"
  goto END
)

echo Generating from 'ApiCOMDoxyConfig'...
"%TOOL_DOXYGEN%" ApiCOMDoxyConfig
if errorlevel 1 (
  set "BATCH_ERR=Doxygen tool failed - ApiCOMDoxyConfig"
  goto END
)


:END
@REM
@REM Did an error occur?
@REM
@if not "%BATCH_ERR%"=="" (
  echo %PROG%:  ERROR: %BATCH_ERR%
  endlocal
  exit /b 1
) else (
  endlocal
)
