#!/bin/bash

#
# For the build environment, certain ENV vars are expected to be 
# setup for tool/config usage.
#    The ENV vars are:  TOOL_DOXYGEN, TOOL_GRAPHVIZ_DIR, CVC_BUILD_VER
#    The default ENV settings may be modified in this script.
#    The CVC_BUILD_VER can be set by passing the version as an arg.
#

VERSION=$1

#
# Check ENV vars and set defaults.
#
if [ -z "${TOOL_DOXYGEN}" ]; then
    # Default install location on Linux & Mac
    export TOOL_DOXYGEN=/usr/local/bin/doxygen
fi
if [ ! -f "${TOOL_DOXYGEN}" ]; then
    echo "ERROR: Could not find ${TOOL_DOXYGEN}"
    exit 1
fi

if [ -z "${TOOL_GRAPHVIZ_DIR}" ]; then
    # Default install location on Mac
    export TOOL_GRAPHVIZ_DIR=/usr/local/bin

    if [ ! -f "${TOOL_GRAPHVIZ_DIR}" ]; then
        # Default install location on linux (rpm)
        export TOOL_GRAPHVIZ_DIR=/usr/bin
    fi
fi
if [ ! -d "${TOOL_GRAPHVIZ_DIR}" ]; then
    echo "ERROR: Could not find ${TOOL_GRAPHVIZ_DIR}"
    exit 1
fi

if [ ! -z "${VERSION}" ]; then
  export CVC_BUILD_VER=$VERSION
fi
if [ -z "${CVC_BUILD_VER}" ]; then
    echo "ERROR: CVC_BUILD_VER is not set"
    exit 1
fi


echo "Generating API docs from 'ApiDoxyConfig'..."
${TOOL_DOXYGEN} ApiDoxyConfig
if [ $? -ne 0 ]; then
    echo "ERROR:  Doxygen tool failed - ApiDoxyConfig"
    exit 1
fi

#
# API-COM is relevant to Windows only
#
#echo "Generating API-COM docs from 'ApiCOMDoxyConfig'..."
#${TOOL_DOXYGEN} ApiCOMDoxyConfig
#if [ $? -ne 0 ]; then
#    echo "ERROR:  Doxygen tool failed - ApiCOMDoxyConfig"
#    exit 1
#fi

exit 0
