/********************************************************************************\
*  This sample is supplied as is with no implied warranty.  
*  It is designed to assist you in using the Cisco AnyConnect VPN API. 
*  It is assumed that you will build a production application and 
*  refer to this sample as a reference only.
\********************************************************************************/
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using VpnApiLib;

namespace CSharpComSample
{
    public partial class StatisticsDlg : Form
    {
        public StatisticsDlg()
        {
            InitializeComponent();
        }

        public void UpdateStats(IVpnStats vpnStats)
        {
            bytesSent.Text = vpnStats[VPNStatsTag.BytesSent];
            bytesReceived.Text = vpnStats[VPNStatsTag.BytesReceived];

            framesSent.Text = vpnStats[VPNStatsTag.PacketsSent];
            framesReceived.Text = vpnStats[VPNStatsTag.PacketsReceived];

            controlFramesSent.Text = vpnStats[VPNStatsTag.ControlPacketsSent];
            controlFramesReceived.Text = vpnStats[VPNStatsTag.ControlPacketsReceived];

            connectState.Text = vpnStats[VPNStatsTag.State];
            connectMode.Text = vpnStats[VPNStatsTag.TunnelingMode];
            connectDuration.Text = vpnStats[VPNStatsTag.TimeConnected];

            addressClient.Text = vpnStats[VPNStatsTag.ClientAddress];
            addressClientV6.Text = vpnStats[VPNStatsTag.ClientAddressV6];
            addressServer.Text = vpnStats[VPNStatsTag.ServerAddress];

            if (vpnStats.ProtocolInfo.Count > 0)
            {
                IProtocolInfo activeProtocol = (IProtocolInfo)vpnStats.ProtocolInfo[1];
                transportCipher.Text = activeProtocol[ProtocolInfoTag.Cipher];
                transportCompression.Text = activeProtocol[ProtocolInfoTag.Compression];
                transportProtocol.Text = activeProtocol[ProtocolInfoTag.Protocol];
            }

            transportProxy.Text = vpnStats[VPNStatsTag.ProxyHostName];

            fipsMode.Text = vpnStats[VPNStatsTag.FipsMode];
            tndState.Text = vpnStats[VPNStatsTag.TrustedNetworkDetectionMode];
            alwaysOn.Text = vpnStats[VPNStatsTag.AlwaysOnMode];
            secureMS.Text = vpnStats[VPNStatsTag.MUSStatus];

            updateRoutes(vpnStats);
            
        }

        private void updateRoutes(IVpnStats vpnStats)
        {
            if (vpnStats.NonsecureRoutes.Count > 0)
            {
                nonSecureRoutesListView.Items.Clear();

                for (int i = 1; i <= vpnStats.NonsecureRoutes.Count; i++)
                {
                    IRouteInfo route = (IRouteInfo)vpnStats.NonsecureRoutes[i];
                    string[] routeDisplay = new string[] { route.Network, route.Subnet };
                    nonSecureRoutesListView.Items.Insert(nonSecureRoutesListView.Items.Count, new ListViewItem(routeDisplay));
                }
            }

            if (vpnStats.SecureRoutes.Count > 0)
            {
                secureRoutesListView.Items.Clear();

                for (int i = 1; i <= vpnStats.SecureRoutes.Count; i++)
                {
                    IRouteInfo route = (IRouteInfo)vpnStats.SecureRoutes[i];
                    string[] routeDisplay = new string[] { route.Network, route.Subnet };
                    secureRoutesListView.Items.Insert(secureRoutesListView.Items.Count, new ListViewItem(routeDisplay));

                }
            }

            updateFirewall(vpnStats);
        }

        private void updateFirewall(IVpnStats vpnStats)
        {
            if (vpnStats.FirewallInfo.Count > 0)
            {
                firewallListView.Items.Clear();

                for (int i = 1; i <= vpnStats.FirewallInfo.Count; i++)
                {
                    IFirewallInfo firewall = (IFirewallInfo)vpnStats.FirewallInfo[i];
                    string[] firewallDisplay = new string[] { firewall.Interface, firewall.Permission, firewall.Protocol, firewall.SrcPortRange, firewall.DstPortRange, firewall.DstAddr };
                    firewallListView.Items.Insert(firewallListView.Items.Count, new ListViewItem(firewallDisplay));
                }

            }

        }

        private void StatisticsDlg_FormClosing(object sender, FormClosingEventArgs e)
        {
            this.Hide();
            e.Cancel = true; // Don't dispose of the form, so it can be re-used later
        }
    }
}